/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.attributes;

import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.common.grpc.NachrichtenCallContextAttachingInterceptor;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAccessPermission;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class ClientAttributeRemoteService {

	@GrpcClient("pluto")
	private ClientAttributeServiceBlockingStub stub;

	public void setBooleanReadOnlyClientAttribute(String vorgangId, String attributeName, boolean value) {
		stub.withInterceptors(new NachrichtenCallContextAttachingInterceptor())
				.set(buildRequest(vorgangId, attributeName, value));
	}

	private GrpcSetClientAttributeRequest buildRequest(String vorgangId, String attributeName, boolean value) {
		return buildRequest(vorgangId, GrpcClientAttribute.newBuilder()
				.setAccess(GrpcAccessPermission.READ_ONLY)
				.setAttributeName(attributeName)
				.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(value).build()).build());
	}

	public void setBooleanWriteableClientAttribute(String vorgangId, String attributeName, boolean value) {
		stub.withInterceptors(new NachrichtenCallContextAttachingInterceptor())
				.set(buildRequestForWritableAttribute(vorgangId, attributeName, value));
	}

	private GrpcSetClientAttributeRequest buildRequestForWritableAttribute(String vorgangId, String attributeName, boolean value) {
		return buildRequest(vorgangId, GrpcClientAttribute.newBuilder()
				.setAccess(GrpcAccessPermission.READ_WRITE)
				.setAttributeName(attributeName)
				.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(value).build()).build());
	}

	private GrpcSetClientAttributeRequest buildRequest(String vorgangId, GrpcClientAttribute attribute) {
		return GrpcSetClientAttributeRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setAttribute(attribute)
				.build();
	}
}
