package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.nachrichten.postfach.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory;

class RueckfrageMapperTest {

	private final RueckfrageMapper mapper = Mappers.getMapper(RueckfrageMapper.class);

	@DisplayName("From postfachNachricht")
	@Nested
	class TestFromPostfachNachricht {

		@Test
		void shouldMapId() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getId()).isEqualTo(PostfachNachrichtTestFactory.ID);
		}

		@Test
		void shouldMapVorgangId() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getVorgangId()).isEqualTo(MessageTestFactory.VORGANG_ID);
		}

		@Test
		void shouldMapPostfachAddress() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getPostfachAddress()).usingRecursiveComparison().isEqualTo(PostfachAddressTestFactory.create());
		}

		@Test
		void shouldMapSentAt() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getSentAt()).isEqualTo(PostfachNachrichtTestFactory.SENT_AT);
		}

		@Test
		void shouldMapText() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getText()).isEqualTo(PostfachNachrichtTestFactory.MAIL_BODY);
		}

		@Test
		void shouldMapAttachments() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getAttachments()).isEqualTo(PostfachNachrichtTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldHaveDefaultStatus() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getStatus()).isEqualTo(RueckfrageStatus.NEW);
		}

		private Rueckfrage mapFromPostfachNachricht() {
			return mapper.fromPostfachNachricht(PostfachNachrichtTestFactory.create());
		}
	}
}
