package de.ozgcloud.nachrichten.info;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.info.nachricht.GrpcNachricht;
import de.ozgcloud.info.nachricht.GrpcNewNachrichtReply;
import de.ozgcloud.info.nachricht.GrpcNewNachrichtRequest;
import de.ozgcloud.info.nachricht.NachrichtServiceGrpc.NachrichtServiceBlockingStub;

class InfoManagerRemoteServiceTest {

	@Spy
	@InjectMocks
	private InfoManagerRemoteService service;

	@Mock
	private NachrichtServiceBlockingStub nachrichtServiceStub;
	@Mock
	private InfomanagerNachrichtMapper nachrichtMapper;

	@Nested
	class TestSendNotification {

		private static final String INFO_MANAGER_STATUS = "status";

		@Mock
		private GrpcNewNachrichtRequest grpcRequest;
		@Mock
		private GrpcNewNachrichtReply grpcReply;

		@BeforeEach
		void setup() {
			doReturn(grpcRequest).when(service).buildNachrichtRequest(any());
			when(nachrichtServiceStub.saveNewNachricht(any())).thenReturn(grpcReply);
		}

		@Test
		void shouldCallGrpcClient() {
			sendNotification();

			verify(nachrichtServiceStub).saveNewNachricht(grpcRequest);
		}

		@Test
		void shouldReturnStatus() {
			when(grpcReply.getStatus()).thenReturn(INFO_MANAGER_STATUS);

			var status = sendNotification();

			assertThat(status).isEqualTo(INFO_MANAGER_STATUS);
		}

		private String sendNotification() {
			return service.sendNotification(InfoManagerRequestTestFactory.create());
		}
	}

	@Nested
	class TestBuildNachrichtRequest {

		@Mock
		private GrpcNachricht grpcNachricht;

		@BeforeEach
		void setup() {
			when(nachrichtMapper.toGrpc(any())).thenReturn(grpcNachricht);
		}

		@Test
		void shouldCallMapper() {
			var nachricht = InfoManagerRequestTestFactory.create();

			service.buildNachrichtRequest(nachricht);

			verify(nachrichtMapper).toGrpc(nachricht);
		}

		@Test
		void shouldSetNachricht() {
			var grpcRequest = service.buildNachrichtRequest(InfoManagerRequestTestFactory.create());

			assertThat(grpcRequest.getNachricht()).isEqualTo(grpcNachricht);
		}
	}
}